--[[-----------------------------------------------------------------------------
ItemInfo Widget
Displays an item icon with name, Reforging, and equipment slot info
-------------------------------------------------------------------------------]]
local Type, Version = "ItemInfo", 1
local AceGUI = LibStub and LibStub("AceGUI-3.0", true)
if not AceGUI or (AceGUI:GetWidgetVersion(Type) or 0) >= Version then return end

-- Lua APIs
local max, select, pairs = math.max, select, pairs

-- WoW APIs
local CreateFrame, UIParent = CreateFrame, UIParent

-- Global vars/functions that we don't upvalue since they might get hooked, or upgraded
-- List them here for Mikk's FindGlobals script
-- GLOBALS: GameFontHighlightSmall

--[[-----------------------------------------------------------------------------
Support functions
-------------------------------------------------------------------------------]]

local function UpdateImageAnchor(self)
	if self.resizing then return end
	local frame = self.frame
	local width = frame.width or frame:GetWidth() or 0
	local image = self.image
	local itemNameLabel = self.itemNameLabel
	local reforgingLabel = self.reforgingLabel
	local equipLocLabel = self.equipLocLabel
	local winnerLabel = self.winnerLabel
	local givenToLabel = self.givenToLabel
	local height

	itemNameLabel:ClearAllPoints()
	reforgingLabel:ClearAllPoints()
	equipLocLabel:ClearAllPoints()
	image:ClearAllPoints()

	local imagewidth = image:GetWidth()
	local imageheight = image:GetHeight()
	-- image on the left
	image:SetPoint("TOPLEFT")
	itemNameLabel:SetPoint("TOPLEFT", image, "TOPRIGHT", 3, 0)
	itemNameLabel:SetWidth(width - imagewidth - 3)
	itemNameLabel:SetHeight(imageheight / 3)
	itemNameLabel:SetFont("Fonts\\FRIZQT__.TTF", imageheight / 3)
	reforgingLabel:SetPoint("TOPLEFT", itemNameLabel, "TOPLEFT", 0, 0 - (imageheight / 3) - 3)
	reforgingLabel:SetWidth(width - imagewidth - 3)
	reforgingLabel:SetHeight(imageheight / 4)
	reforgingLabel:SetFont("Fonts\\FRIZQT__.TTF", imageheight / 4)
	equipLocLabel:SetPoint("TOPLEFT", itemNameLabel, "TOPLEFT", 0, 0 - (imageheight / 3) - (imageheight / 4) - 6)
	equipLocLabel:SetWidth(width - imagewidth - 3)
	equipLocLabel:SetHeight(imageheight / 4)
	equipLocLabel:SetFont("Fonts\\FRIZQT__.TTF", imageheight / 4)
	winnerLabel:SetPoint("TOP", equipLocLabel)
	winnerLabel:SetPoint("RIGHT", frame)
	winnerLabel:SetHeight(imageheight / 4)
	winnerLabel:SetFont("Fonts\\FRIZQT__.TTF", imageheight / 4)
	--winnerLabel:SetWidth(winnerLabel:GetStringWidth())
	givenToLabel:SetPoint("TOPRIGHT", winnerLabel, "TOPLEFT")
	givenToLabel:SetHeight(imageheight / 4)
	givenToLabel:SetFont("Fonts\\FRIZQT__.TTF", imageheight / 4)
	height = max(image:GetHeight(), itemNameLabel:GetHeight() + reforgingLabel:GetHeight() + equipLocLabel:GetHeight() + 6)
	
	self.resizing = true
	frame:SetHeight(height)
	self.height = height
	self.width = frame:GetWidth()
	self.resizing = nil
end

--[[-----------------------------------------------------------------------------
Methods
-------------------------------------------------------------------------------]]
local methods = {
	["OnAcquire"] = function(self)
		-- set the flag to stop constant size updates
		self.resizing = true
		-- height is set dynamically by the text and image size
		self:SetWidth(200)
		self:SetNameText()
		self:SetReforgingText()
		self:SetEquipLocText()
		self:SetImage(nil)
		self:SetImageSize(62, 62)
		self:SetColor()

		-- reset the flag
		self.resizing = nil
		-- run the update explicitly
		UpdateImageAnchor(self)
	end,

	["OnWidthSet"] = function(self, width)
		UpdateImageAnchor(self)
	end,

	["SetNameText"] = function(self, text)
		self.itemNameLabel:SetText(text)
		UpdateImageAnchor(self)
	end,
	
	["SetLink"] = function(self, link)
		local frame = self.frame
		self.imageButton:SetScript("OnEnter", function()
			GameTooltip:SetOwner(frame, "ANCHOR_BOTTOMLEFT")
			GameTooltip:SetHyperlink(link)
			GameTooltip:Show()	
		end)
		self.imageButton:SetScript("OnClick", function()
			ShowUIPanel(ItemRefTooltip)
			if (not ItemRefTooltip:IsVisible()) then
				ItemRefTooltip:SetOwner(UIParent, "ANCHOR_PRESERVE");
			end
			ItemRefTooltip:SetHyperlink(link)
			ItemRefTooltip:Show()
		end)
	end,

	["SetReforgingText"] = function(self, text)
		self.reforgingLabel:SetText(text)
		UpdateImageAnchor(self)
	end,

	["SetEquipLocText"] = function(self, text)
		self.equipLocLabel:SetText(text)
		UpdateImageAnchor(self)
	end,

	["SetWinnerText"] = function(self, labelText, text, class)
		if text and text ~= "" then
			self.givenToLabel:SetText(labelText .. ": ")
			self.winnerLabel:SetText(text)
			self.winnerLabel:SetTextColor(RAID_CLASS_COLORS[class].r, RAID_CLASS_COLORS[class].g, RAID_CLASS_COLORS[class].b)
		else
			self.givenToLabel:SetText("")
			self.winnerLabel:SetText("")
		end
		UpdateImageAnchor(self)
	end,

	["SetColor"] = function(self, r, g, b)
		if not (r and g and b) then
			r, g, b = 1, 1, 1
		end
		self.itemNameLabel:SetVertexColor(r, g, b)
	end,

	["SetImage"] = function(self, path, ...)
		local image = self.image
		image:SetTexture(path)
		
		if image:GetTexture() then
			local n = select("#", ...)
			if n == 4 or n == 8 then
				image:SetTexCoord(...)
			else
				image:SetTexCoord(0, 1, 0, 1)
			end
		end
		UpdateImageAnchor(self)
	end,

	["SetImageSize"] = function(self, width, height)
		self.image:SetWidth(width)
		self.image:SetHeight(height)
		UpdateImageAnchor(self)
	end,
}

--[[-----------------------------------------------------------------------------
Constructor
-------------------------------------------------------------------------------]]
local function Constructor()
	local frame = CreateFrame("Frame", nil, UIParent)
	frame:Hide()

	local itemNameLabel = frame:CreateFontString(nil, "BACKGROUND", "GameFontHighlightSmall")
	itemNameLabel:SetJustifyH("LEFT")
	itemNameLabel:SetJustifyV("TOP")

	local reforgingLabel = frame:CreateFontString(nil, "BACKGROUND", "GameFontHighlightSmall")
	reforgingLabel:SetJustifyH("LEFT")
	reforgingLabel:SetJustifyV("TOP")
	reforgingLabel:SetVertexColor(GetItemQualityColor(2))
	
	local equipLocLabel = frame:CreateFontString(nil, "BACKGROUND", "GameFontHighlightSmall")
	equipLocLabel:SetJustifyH("LEFT")
	equipLocLabel:SetJustifyV("TOP")
	
	local winnerLabel = frame:CreateFontString(nil, "BACKGROUND", "GameFontHighlightSmall")
	winnerLabel:SetJustifyH("RIGHT")
	winnerLabel:SetJustifyV("TOP")
	
	local givenToLabel = frame:CreateFontString(nil, "BACKGROUND", "GameFontHighlightSmall")
	givenToLabel:SetJustifyH("RIGHT")
	givenToLabel:SetJustifyV("TOP")
	givenToLabel:SetVertexColor(YELLOW_FONT_COLOR.r, YELLOW_FONT_COLOR.g, YELLOW_FONT_COLOR.b)
	
	local image = frame:CreateTexture(nil, "BACKGROUND")

	local imageButton = CreateFrame("Button", nil, frame)
	imageButton:SetFrameStrata("FULLSCREEN_DIALOG")
	imageButton:SetAllPoints(image)
	imageButton:EnableMouse(true)
	imageButton:SetScript("OnLeave", function() GameTooltip:Hide() end)

	-- create widget
	local widget = {
		itemNameLabel = itemNameLabel,
		reforgingLabel = reforgingLabel,
		equipLocLabel = equipLocLabel,
		winnerLabel = winnerLabel,
		givenToLabel = givenToLabel,
		image = image,
		imageButton = imageButton,
		frame = frame,
		type  = Type
	}
	for method, func in pairs(methods) do
		widget[method] = func
	end

	return AceGUI:RegisterAsWidget(widget)
end

AceGUI:RegisterWidgetType(Type, Constructor, Version)
